# packages
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from enum import Enum
from typing import Tuple, List

# stats
from statsmodels.tsa.ar_model import AutoReg
from statsmodels.graphics.tsaplots import plot_pacf, acf

# qis
import qis.file_utils as fu
import qis.perfstats.returns as ret
import qis.plots.time_series as pts
from qis.plots.derived.prices import plot_prices

# internal
import qis.models.stats.bootstrap as bts


def plot_price_bootrstrap(prices: pd.Series,
                          seed: int = 1,
                          block_size: int = 20,
                          **kwargs):
    bt_data = bts.bootstrap_price_data(prices=prices,
                                       bootsrap_type=bts.BootsrapType.STATIONARY,
                                       bootsrap_output=bts.BootsrapOutput.SERIES_TO_DF,
                                       index_length=len(prices),
                                       seed=seed,
                                       block_size=block_size)
    bt_data.index = prices.index
    all_data = pd.concat([prices, bt_data], axis=1)
    plot_prices(all_data, **kwargs)


def plot_ew_index_bootrstrap(prices: pd.DataFrame):
    log_returns = ret.to_returns(prices=prices, is_log_returns=True, drop_first=True)
    bt_data = bts.bootstrap_data(data=log_returns,
                                 bootsrap_type=bts.BootsrapType.STATIONARY,
                                 bootsrap_output=bts.BootsrapOutput.DF_TO_LIST_ARRAYS,
                                 index_length=len(prices))
    ew_data = []
    for idx, sample in enumerate(bt_data):
        ew_data.append(pd.Series(np.nanmean(sample, axis=1), name=f"path_{idx+1}"))
    ew_data = pd.concat(ew_data, axis=1)
    ew_data.index = prices.index
    ew_prices = ret.returns_to_nav(ew_data)
    original = ret.returns_to_nav(
        ret.to_returns(prices=prices, is_log_returns=True).mean(1)).rename('realized')
    all_data = pd.concat([original, ew_prices], axis=1)
    plot_prices(all_data)


def estimate_ar_model(data: pd.Series,
                      title: str = None,
                      axs: List[plt.Subplot] = None,
                      **kwargs):
    ar_model = AutoReg(data, lags=1).fit()
    print(ar_model.summary())

    intercept, beta = ar_model.params[0], ar_model.params[1]
    prediction = np.zeros(len(data.index))
    prediction[0] = data[0]
    for t in np.arange(1, len(data.index)):
        prediction[t] = beta*data[t-1] + intercept

    prediction = pd.Series(prediction, index=data.index, name='prediction')
    joint_data = pd.concat([data, prediction], axis=1)

    if axs is None:
        with sns.axes_style('darkgrid'):
            fig, axs = plt.subplots(2, 1, figsize=(12, 9), tight_layout=True)

    plot_pacf(data, lags=10, title=f"{title}: Partial Auto-Correlation", ax=axs[0])
    out = acf(data)
    print(out)

    pts.plot_time_series(df=joint_data,
                         legend_stats=pts.LegendStats.AVG_STD,
                         var_format='{:,.0f}',
                         title=f"{title}: One-Day Prediction",
                         ax=axs[1],
                         **kwargs)


def plot_ar_bootstrap(data: pd.Series,
                      title: str = None,
                      ax: plt.Subplot = None,
                      num_samples: int = 10,
                      seed: int = 2,
                      **kwargs
                      ):

    df_bts = bts.bootstrap_ar_process(data=data,
                                      bootsrap_output=bts.BootsrapOutput.SERIES_TO_DF,
                                      index_length=len(data.index),
                                      num_samples=num_samples,
                                      seed=seed)
    df_bts.index = data.index
    joint_data = pd.concat([data.rename('realized'), df_bts], axis=1)
    pts.plot_time_series(df=joint_data,
                         legend_stats=pts.LegendStats.AVG_STD,
                         var_format='{:,.0f}',
                         title=title,
                         y_limits=(0.0, None),
                         first_color_fixed=False,
                         ax=ax,
                         **kwargs)


def plot_joint_bootstrap():
    prices, fundamental_data = get_test_data()
    prices, fundamental_data = prices.iloc[:, 0], fundamental_data.iloc[:, 0]
    price_datas = {'price': prices.iloc[:, 0]}
    fundamental_datas = {'fundamental': fundamental_data}
    bootstrap_prices, bootstrap_fundamentals = bts.bootstrap_price_fundamental_data(price_datas=price_datas,
                                                                                    fundamental_datas=fundamental_datas,
                                                                                    bootsrap_output=bts.BootsrapOutput.SERIES_TO_DF,
                                                                                    index_length=len(prices.index),
                                                                                    num_samples=10,
                                                                                    block_size=10)
    bootstrap_prices = bootstrap_prices['price']
    bootstrap_prices.index = prices.index
    bootstrap_fundamentals = bootstrap_fundamentals['fundamental']
    bootstrap_fundamentals.index = prices.index
    join_price_data = pd.concat([prices, bootstrap_prices], axis=1)
    joint_fund_data = pd.concat([fundamental_data.rename('realized'), bootstrap_fundamentals], axis=1)

    with sns.axes_style('darkgrid'):
        fig, axs = plt.subplots(2, 1, figsize=(12, 10), tight_layout=True)

        pts.plot_time_series(df=joint_fund_data,
                             legend_stats=pts.LegendStats.AVG_STD,
                             var_format='{:,.0f}',
                             first_color_fixed=True,
                             ax=axs[0])
        plot_prices(prices=join_price_data,
                        is_log=True,
                        ax=axs[1])


def get_test_data() -> Tuple[pd.DataFrame, pd.DataFrame]:
    tickers = ['uniswap_msr', 'terra_msr']
    price_datas = []
    fund_datas = []
    for ticker in tickers:
        data = fu.load_df_from_csv(file_name=ticker, folder_name='crypto')
        price_datas.append(data['close'].rename(ticker))
        fund_datas.append(np.divide(data['volume'], data['close']).rename(ticker))
    prices = pd.concat(price_datas, axis=1).dropna()
    fund_data = pd.concat(fund_datas, axis=1).dropna()
    prices = prices.reindex(index=fund_data.index, method='ffill')
    return prices, fund_data


class UnitTests(Enum):
    PLOT_PRICE_BOOTSTRAP = 1
    PLOT_EW_INDEX = 2
    FIT_AR_MODEL = 3
    AR_RESIDUALS = 4
    PLOT_AR_BOOTSTRAP = 5
    PLOT_JOINT_BOOTSTRAP = 6


def run_unit_test(unit_test: UnitTests):

    from qis.test_data import load_etf_data
    prices = load_etf_data().dropna()

    if unit_test == UnitTests.PLOT_PRICE_BOOTSTRAP:
        plot_price_bootrstrap(prices['SPY'])

    elif unit_test == UnitTests.PLOT_EW_INDEX:
        plot_ew_index_bootrstrap(prices)

    elif unit_test == UnitTests.FIT_AR_MODEL:
        data = fu.load_df_from_csv(file_name='UNI_Uniswap_cmc', folder_name='crypto/instruments')
        unit_volume = np.divide(data['volume'], data['close']).rename('unit volume')
        estimate_ar_model(data=unit_volume)

    elif unit_test == UnitTests.AR_RESIDUALS:
        prices, fund_data = get_test_data()
        residuals, intercept, beta = bts.compute_ar_residuals(data=fund_data)
        residuals = pd.DataFrame(residuals, index=fund_data.index, columns=fund_data.columns)
        pts.plot_time_series(residuals,
                             legend_stats=pts.LegendStats.AVG_STD,
                             var_format='{:,.0f}')

    elif unit_test == UnitTests.PLOT_AR_BOOTSTRAP:
        prices, fund_data = get_test_data()
        data = fund_data.iloc[:, 1]
        plot_ar_bootstrap(data=data)

    elif unit_test == UnitTests.PLOT_JOINT_BOOTSTRAP:
        plot_joint_bootstrap()

    plt.show()


if __name__ == '__main__':

    unit_test = UnitTests.FIT_AR_MODEL

    is_run_all_tests = False
    if is_run_all_tests:
        for unit_test in UnitTests:
            run_unit_test(unit_test=unit_test)
    else:
        run_unit_test(unit_test=unit_test)



# buit in
import numpy as np
import pandas as pd
import statsmodels.api as sm
from enum import Enum
from typing import Dict

# qis
from qis.perfstats.regime_classifier import BenchmarkReturnsQuantileRegimeSpecs, BenchmarkReturnsQuantilesRegime


RESID_VAR_COLUMN = 'An var resid'
ALPHA_COLUMN = 'Alpha'
COLOR_COLUMN = 'Color'
PREDICTION = 'Prediction'
VAR = 'Var'


def estimate_cond_regression(prices: pd.DataFrame,
                             benchmark: str,
                             regime_params: BenchmarkReturnsQuantileRegimeSpecs = None,
                             is_print_summary: bool = False,
                             drop_benchmark: bool = True,  # in some regressions we need to keep the benchmark inside
                             min_period: int = 0,  # min number of return in y
                             is_add_alpha: bool = False,
                             regmodel_out_dict: Dict[str, pd.DataFrame] = None
                             ) -> pd.DataFrame:

    """
    estimate beta params for multiple assets
    sampled_returns_with_regime include regime columns
    assets may have nan, if all returns are non -> zero is returnded
    important is that benchmark is fist: if it fails there, the benchmark is has nons which is badly defined
    """
    if regime_params is None:
        regime_params = BenchmarkReturnsQuantileRegimeSpecs()
    regime_classifier = BenchmarkReturnsQuantilesRegime(regime_params=regime_params)
    sampled_returns_with_regime_id = regime_classifier.compute_sampled_returns_with_regime_id(prices=prices,
                                                                                              benchmark=benchmark,
                                                                                              **regime_params._asdict())
    sampled_returns_with_regime_id = sampled_returns_with_regime_id.dropna(how='all', axis=0)
    regime_groups = sampled_returns_with_regime_id.groupby([BenchmarkReturnsQuantilesRegime.REGIME_COLUMN], observed=False)
    # create prediction matrix
    regression_datas = []
    for regime in regime_classifier.get_regime_ids():
        # regime-conditional benchmark data will be benchmark returns renamed by regime name
        benchmark_data = regime_groups.get_group((regime, ))[benchmark].rename(regime)
        regression_datas.append(benchmark_data)

    # now x is 3 column manrix with bear normal bull columns
    regression_matrix = pd.concat(regression_datas, axis=1).fillna(0)
    if is_add_alpha:
        regression_matrix_1 = sm.add_constant(regression_matrix).rename(columns={'const': ALPHA_COLUMN})
    else:
        regression_matrix_1 = regression_matrix

    # drop regime column to leave only assets
    if drop_benchmark:
        asset_returns = sampled_returns_with_regime_id.drop(columns=[BenchmarkReturnsQuantilesRegime.REGIME_COLUMN, benchmark])
    else:
        asset_returns = sampled_returns_with_regime_id.drop(columns=[BenchmarkReturnsQuantilesRegime.REGIME_COLUMN])

    regime_colors = regime_classifier.class_data_to_colors(sampled_returns_with_regime_id[BenchmarkReturnsQuantilesRegime.REGIME_COLUMN]).rename(COLOR_COLUMN)
    model_params = []
    for asset in asset_returns:  # fit model

        regression_matrix_a = regression_matrix_1
        y = asset_returns[asset]
        nan_ind = np.isnan(y.to_numpy())
        if np.all(nan_ind):
            if len(model_params) == 0:
                raise ValueError(f"the first y has all nans, which must be benchmark")
            else:
                estimated_model_params = pd.Series(data=np.nan, index=model_params[0].index)
        else:
            if np.sum(nan_ind == False) < min_period: # skip estimation
                estimated_model_params = pd.Series(data=np.nan, index=model_params[0].index)
            else:
                if np.any(nan_ind):
                    y = y.loc[nan_ind==False]
                # align with y index
                regression_matrix_a = regression_matrix_1.loc[y.index, :]
                model = sm.OLS(y, regression_matrix_a)
                estimated_model = model.fit()

                if is_print_summary:
                    print(estimated_model.summary())
                if regmodel_out_dict is not None:
                    prediction = estimated_model.predict(regression_matrix_1)
                    pandas_out = pd.concat([sampled_returns_with_regime_id[BenchmarkReturnsQuantilesRegime.REGIME_COLUMN],
                                            sampled_returns_with_regime_id[benchmark],
                                            regression_matrix,
                                            y,
                                            prediction.rename(PREDICTION),
                                            regime_colors
                                            ], axis=1)
                    regmodel_out_dict[asset] = pandas_out

                estimated_model_params = estimated_model.params.copy()
                estimated_model_params[RESID_VAR_COLUMN] = estimated_model.mse_resid

        # append to all assets params
        model_params.append(estimated_model_params.rename(asset))

    model_params = pd.concat(model_params, axis=1).T

    return model_params


def get_regime_regression_params(prices: pd.DataFrame,
                                 regime_params: BenchmarkReturnsQuantileRegimeSpecs,
                                 benchmark: str,
                                 is_print_summary: bool = True,
                                 drop_benchmark: bool = False,
                                 min_period: int = 0,
                                 ) -> pd.DataFrame:

    estimated_params = estimate_cond_regression(prices=prices,
                                                benchmark=benchmark,
                                                regime_params=regime_params,
                                                is_print_summary=is_print_summary,
                                                drop_benchmark=drop_benchmark,
                                                min_period=min_period)
    return estimated_params


class UnitTests(Enum):
    REGRESSION = 1


def run_unit_test(unit_test: UnitTests):

    from qis.test_data import load_etf_data
    prices = load_etf_data().dropna()

    print(prices)

    if unit_test == UnitTests.REGRESSION:

        estimated_params = get_regime_regression_params(prices=prices,
                                                        regime_params=BenchmarkReturnsQuantileRegimeSpecs(),
                                                        benchmark='SPY',
                                                        drop_benchmark=True,
                                                        is_print_summary=True)
        print(estimated_params)


if __name__ == '__main__':

    unit_test = UnitTests.REGRESSION

    is_run_all_tests = False
    if is_run_all_tests:
        for unit_test in UnitTests:
            run_unit_test(unit_test=unit_test)
    else:
        run_unit_test(unit_test=unit_test)



"""
analytics for computing portfolio risk with ewm filter
"""

import warnings
import numpy as np
import pandas as pd
from numba import njit
from typing import Union, Optional, List, Tuple

import qis.utils.dates as da
import qis.utils.np_ops as npo
import qis.utils.df_groups as dfg
import qis.utils.df_agg as dfa
import qis.perfstats.returns as ret
from qis.models.linear.ewm import MeanAdjType, InitType, NanBackfill, compute_rolling_mean_adj, compute_ewm_vol


VAR99 = 2.3263
VAR99_SCALER_BP = VAR99 * 10000


def compute_portfolio_vol(returns: pd.DataFrame,
                          weights: pd.DataFrame,
                          span: Union[int, np.ndarray] = None,
                          ewm_lambda: Union[float, np.ndarray] = 0.94,
                          is_return_vol: bool = True,
                          mean_adj_type: MeanAdjType = MeanAdjType.NONE,
                          init_type: InitType = InitType.ZERO,
                          annualize: bool = False,
                          annualization_factor: float = None,
                          nan_backfill: NanBackfill = NanBackfill.FFILL
                          ) -> pd.Series:
    """
    compute portfolio vol using ewm filter
    """
    # align index and columns
    weights, returns = weights.align(other=returns, join='inner')
    weights = weights.shift(1)

    returns_np = npo.to_finite_np(data=returns, fill_value=0.0)
    weights_np = npo.to_finite_np(data=weights, fill_value=0.0)

    if mean_adj_type != MeanAdjType.NONE:
        returns_np = compute_rolling_mean_adj(data=returns_np,
                                              mean_adj_type=mean_adj_type,
                                              span=span,
                                              ewm_lambda=ewm_lambda,
                                              init_type=init_type,
                                              nan_backfill=nan_backfill)

    portfolio_vol = compute_portfolio_vol_np(returns=returns_np,
                                             weights=weights_np,
                                             span=span,
                                             ewm_lambda=ewm_lambda)

    if annualize:
        if annualization_factor is None:
            if isinstance(weights, pd.DataFrame):
                annualization_factor = da.infer_an_from_data(data=weights)
            else:
                warnings.warn(f"in compute_ewm: annualization_factor for np array, default is 1")
                annualization_factor = 1.0

        portfolio_vol = annualization_factor * portfolio_vol

    if is_return_vol:
        portfolio_vol = np.sqrt(portfolio_vol)

    portfolio_vol = pd.Series(data=portfolio_vol, index=weights.index)

    return portfolio_vol


@njit
def compute_portfolio_vol_np(returns: np.ndarray,
                             weights: np.ndarray,
                             span: Union[int, np.ndarray] = None,
                             ewm_lambda: Union[float, np.ndarray] = 0.94
                             ) -> np.ndarray:

    t = returns.shape[0]  # time dimension
    n = returns.shape[1]  # space dimension

    # important to replace nans for @ operator
    weights = np.where(np.isfinite(weights), weights, 0.0)
    last_covar = np.zeros((n, n))
    portfolio_vol = np.zeros(t)
    if span is not None:
        ewm_lambda = 1.0 - 2.0 / (span + 1.0)
    ewm_lambda_1 = 1.0 - ewm_lambda
    for idx in range(0, t):  # row in x
        row = returns[idx]
        r_ij = np.outer(row, row)
        covar = ewm_lambda_1 * r_ij + ewm_lambda * last_covar
        covar = np.where(np.isfinite(covar), covar, ewm_lambda*last_covar)
        last_covar = covar
        weights_t = np.ascontiguousarray(weights[idx])  # to remove NumbaPerformanceWarning warning
        portfolio_vol[idx] = weights_t.T @ covar @ weights_t

    return portfolio_vol


def compute_portfolio_correlated_var_by_ac(prices: pd.DataFrame,
                                           weights: pd.DataFrame,
                                           group_data: Optional[pd.Series] = None,
                                           group_order: List[str] = None,
                                           total_column: Optional[str] = 'Total',
                                           freq: Optional[str] = 'B',
                                           vol_span: int = 31,  # span in number of freq-retunrs
                                           time_period: da.TimePeriod = None,
                                           mean_adj_type: MeanAdjType = MeanAdjType.NONE
                                           ) -> pd.DataFrame:
    """
    portfolio VAR accounting for correlations
    by the default var is max daily loss in bp
    if group_data is provided the var is split into groups
    """
    returns = ret.to_returns(prices=prices, freq=freq, is_log_returns=True)

    weights, returns = weights.align(other=returns, join='inner')
    # weights = weights.shift(1)

    if group_data is not None:
        ac_exposures_dict = dfg.split_df_by_groups(df=weights, group_data=group_data, group_order=group_order,
                                                   total_column=total_column)
        ac_returns_dict = dfg.split_df_by_groups(df=returns, group_data=group_data, group_order=group_order,
                                                 total_column=total_column)
    else:
        ac_exposures_dict = {'Total VAR': weights}
        ac_returns_dict = {'Total VAR': returns}

    portfolio_vars = {}
    for (ac, ac_exposure), (ac, ac_returns) in zip(ac_exposures_dict.items(), ac_returns_dict.items()):
        portfolio_vars[ac] = compute_portfolio_vol(returns=ac_returns,
                                                   weights=ac_exposure,
                                                   span=vol_span,
                                                   mean_adj_type=mean_adj_type,
                                                   annualize=False)
    portfolio_vars = pd.DataFrame.from_dict(portfolio_vars, orient='columns')
    portfolio_vars = VAR99*portfolio_vars
    if time_period is not None:
        portfolio_vars = time_period.locate(portfolio_vars)
    return portfolio_vars


def compute_portfolio_independent_var_by_ac(prices: pd.DataFrame,
                                            weights: pd.DataFrame,
                                            group_data: Optional[pd.Series] = None,
                                            group_order: List[str] = None,
                                            total_column: Optional[str] = 'Total',
                                            freq: Optional[str] = 'B',
                                            vol_span: int = 31,  # span in number of freq-retunrs
                                            time_period: da.TimePeriod = None,
                                            mean_adj_type: MeanAdjType = MeanAdjType.NONE
                                            ) -> Tuple[pd.DataFrame, pd.DataFrame]:
    """
    portfolio VAR accounting for correlations
    by the default var is max daily loss in bp
    if group_data is provided the var is split into groups
    """
    returns = ret.to_returns(prices=prices, freq=freq, is_log_returns=True)
    vols = compute_ewm_vol(data=returns,
                           span=vol_span,
                           mean_adj_type=mean_adj_type,
                           annualize=False)

    weights, vols = weights.align(other=vols, join='inner')

    instrument_vars = VAR99 * vols.mul(np.abs(weights))

    if group_data is not None:
        ac_vars = dfg.agg_df_by_groups(df=instrument_vars,
                                       group_data=group_data,
                                       group_order=group_order,
                                       agg_func=dfa.nansum,
                                       total_column=total_column)
    else:
        ac_vars = instrument_vars.sum(1)

    if time_period is not None:
        instrument_vars = time_period.locate(instrument_vars)
        ac_vars = time_period.locate(ac_vars)
    return instrument_vars, ac_vars


"""
useful as interface when data can be either price or level time series
"""
import pandas as pd
from typing import Optional, Union
import matplotlib.pyplot as plt

# qis
import qis.utils.dates as da
from qis.perfstats.config import PerfParams
import qis.plots.time_series as pts
import qis.plots.derived.prices as ppd


PERF_PARAMS = PerfParams(freq_reg='B', freq_vol='B', freq_drawdown='B')


def plot_data_timeseries(data: Union[pd.DataFrame, pd.Series],
                         is_price_data: bool = False,
                         legend_stats: pts.LegendStats = pts.LegendStats.FIRST_AVG_LAST,
                         var_format: Optional[str] = None,
                         time_period: da.TimePeriod = None,
                         title: str = '',
                         title_add_date: bool = True,
                         perf_params: PerfParams = PERF_PARAMS,
                         start_to_one: bool = False,
                         ax: plt.Subplot = None,
                         **kwargs
                         ) -> None:
    """
    define plot time series
    """
    if time_period is not None:
        data = time_period.locate(data)

    if title_add_date:
        title = f"{title} {da.get_time_period(df=data.dropna()).to_str()}"
    else:
        title = title

    if var_format is None:
        if is_price_data:
            var_format = '{:,.2f}'
        else:
            var_format = '{:.2%}'

    if is_price_data:
        ppd.plot_prices(prices=data,
                        perf_params=perf_params,
                        title=title,
                        start_to_one=start_to_one,
                        is_log=False,
                        var_format=var_format,
                        perf_stats_labels=ppd.PerfStatsLabels.DETAILED_WITH_DDVOL.value,
                        ax=ax,
                        **kwargs)
    else:
        pts.plot_time_series(df=data,
                             title=title,
                             legend_stats=legend_stats,
                             var_format=var_format,
                             ax=ax,
                             **kwargs)



"""
analyse performance of CBOE vol strats
use downloaded data
"""

# imports
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import yfinance as yf
import qis


# download from https://cdn.cboe.com/api/global/us_indices/daily_prices/SVRPO_History.csv
svrpo = qis.load_df_from_csv(file_name='SVRPO_History', local_path=qis.get_resource_path())
# spy etf as benchmark
benchmark = 'SPY'
spy = yf.download([benchmark], start=None, end=None)['Adj Close'].rename(benchmark)
# merge
prices = pd.concat([spy, svrpo], axis=1).dropna()
# take last 3 years
prices = prices.loc['2020':, :]

# set parameters for computing performance stats including returns vols and regressions
ust_3m_rate = yf.download('^IRX', start=None, end=None)['Adj Close'].dropna() / 100.0
perf_params = qis.PerfParams(freq='ME', freq_reg='W-WED', alpha_an_factor=52.0, rates_data=ust_3m_rate)

# price perf
with sns.axes_style("darkgrid"):
    fig1, axs = plt.subplots(2, 1, figsize=(10, 7))
    qis.plot_prices_with_dd(prices=prices,
                            regime_benchmark_str=benchmark,
                            x_date_freq='QE',
                            framealpha=0.9,
                            perf_params=perf_params,
                            axs=axs)
    fig2, ax = plt.subplots(1, 1, figsize=(10, 7))
    qis.plot_returns_scatter(prices=prices,
                             benchmark=benchmark,
                             ylabel=svrpo.columns[0],
                             title='Regression of weekly returns',
                             freq='W-WED',
                             ax=ax)


plt.show()


import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import qis as qis
from bbg_fetch import fetch_field_timeseries_per_tickers

vix_tickers = ['VIX1D Index', 'VIX9D Index', 'VIX Index', 'VIX3M Index', 'VIX6M Index', 'VIX1Y Index']

vols = 0.01*fetch_field_timeseries_per_tickers(tickers=vix_tickers, field='PX_LAST', CshAdjNormal=True).dropna()
benchmark = 'SPX Index'
snp = fetch_field_timeseries_per_tickers(tickers=[benchmark], field='PX_LAST', CshAdjNormal=True).dropna()
snp = snp.reindex(index=vols.index, method='ffill')
df = pd.concat([snp.pct_change(), vols.diff(1)], axis=1).dropna()

with sns.axes_style("darkgrid"):
    # qis.plot_time_series(df=vols, ax=axs[0])
    # qis.plot_prices(prices=snp, ax=axs[0])
    kwargs = dict(fontsize=12, framealpha=0.75)

    corr = qis.compute_masked_covar_corr(data=df, is_covar=False)
    fig1, ax = plt.subplots(1, 1, figsize=(4, 4))
    qis.plot_heatmap(df=corr,
                     var_format='{:.0%}',
                     cmap='PiYG',
                     title=f"Correlation between daily changes for period {qis.get_time_period_label(df, date_separator='-')}",
                     fontsize=12,
                     ax=ax)
    """
    fig1, axs = plt.subplots(2, 1, figsize=(18, 9))
    qis.plot_scatter(df=df,
                     x=benchmark,
                     xlabel=f'{benchmark} daily return',
                     ylabel=f'Vix daily change',
                     title=f'Daily change in VIX indices predicted by daily return of benchmark',
                     xvar_format='{:.0%}',
                     yvar_format='{:.0%}',
                     order=2,
                     fit_intercept=True,
                     add_hue_model_label=True,
                     ci=95,
                     ax=axs[0],
                     **kwargs)
    """
    fig2, axs = plt.subplots(2, 3, figsize=(18, 9))
    axs = qis.to_flat_list(axs)
    for idx, vol in enumerate(vols.columns):
        qis.plot_classification_scatter(df=df[[benchmark, vol]],
                                        full_sample_order=1,
                                        num_buckets=2,
                                        xvar_format='{:.0%}',
                                        yvar_format='{:.0%}',
                                        fit_intercept=True,
                                        title=f"{vol}",
                                        ax=axs[idx],
                                        **kwargs)

plt.show()
